import 'package:flutter/material.dart';
import 'package:insurance_card_scanner/src/models/card_scan_messages.dart';
import 'package:insurance_card_scanner/src/models/scanned_card.dart';
import 'package:insurance_card_scanner/src/utils/logger.dart';

typedef ErrorCallback = Function(String?);
typedef ScanSuccessCallback = Function(ScannedCard);

/// Properties and callbacks set to CardScanner
class CardScanConfig {
  const CardScanConfig({
    required this.sessionToken,
    required this.onSuccess,
    this.logging = LogLevel.normal,
    this.live = false,
    this.onCancel,
    this.onError,
    this.backsideSupport = false,
    this.onRetry,
    this.messages,
    this.messageStyle,
    this.autoSwitchActiveColor,
    this.autoSwitchInactiveColor,
    this.progressBarColor,
    this.widgetBackgroundColor,
    this.onWebViewLoadError,
  });

  /// JWT used for authenticating users during CardScanner API requests.
  ///
  /// See more: https://docs.cardscan.ai/authentication
  final String sessionToken;

  /// Callback on successful card scanning. Returns a [ScannedCard] object when
  /// triggered.
  final ScanSuccessCallback? onSuccess;

  /// Flag for choosing the level of detail for logs,
  /// choose between off, normal, verbose, detector or everything.
  final LogLevel logging;

  /// Flag for current environment of the card scanner.
  ///
  /// If `true`, the production environment will be set. Otherwise, the sandbox
  /// environment will be used.
  final bool live;

  /// Callback triggered when user cancels/closes the CardScanner UI.
  final VoidCallback? onCancel;

  /// Callback triggered on potential errors. Errors happend when calling the
  /// API or when CardScanner fails to initialize.
  final ErrorCallback? onError;

  /// Enable scanning of the front and back side of the card.
  ///
  /// If set to false, the CardScanner will only expect to scan the front side
  /// of a card.
  final bool backsideSupport;

  /// Callback triggered when a scan fails.
  final VoidCallback? onRetry;

  /// Custom texts for giving visual feedback inside CardScanner UI.
  final CardScanMessages? messages;

  /// Adjust text properties to the message prompted by the UI.
  final TextStyle? messageStyle;

  /// Enabled color for the auto scan switch.
  final Color? autoSwitchActiveColor;

  /// Disabled color for the auto scan switch.
  final Color? autoSwitchInactiveColor;

  /// Color for the progress bar and bounding box surrounding the card scan area.
  final Color? progressBarColor;

  /// Main background color for the widget.
  final Color? widgetBackgroundColor;

  final VoidCallback? onWebViewLoadError;
}
