import 'dart:convert';
import 'dart:io';

import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_inappwebview/flutter_inappwebview.dart';
import 'package:insurance_card_scanner/src/models/models.dart';
import 'package:insurance_card_scanner/src/utils/js_injection.dart';
import 'package:insurance_card_scanner/src/utils/logger.dart';
import 'package:pubspec_parse/pubspec_parse.dart';

const _liveServer = 'api.cardscan.ai';

const _dev = bool.fromEnvironment('CARDSCAN_DEV');

const _sandboxServer = String.fromEnvironment(
  'CARDSCAN_AI_SERVER',
  defaultValue: !_dev ? 'sandbox.cardscan.ai' : 'dev.nop.sh',
);

class ScannerWebView extends StatefulWidget {
  const ScannerWebView({super.key, required this.properties});

  final CardScanConfig properties;

  @override
  State<ScannerWebView> createState() => _ScannerWebViewState();
}

class _ScannerWebViewState extends State<ScannerWebView> {
  final webViewKey = GlobalKey();

  String get webViewUrl =>
      widget.properties.live ? _liveServer : _sandboxServer;

  /// Retrieves version for the insurance_card_scanner package
  Future<String> getPackageVersion() async {
    final fileContent = await rootBundle.loadString(
      "packages/insurance_card_scanner/pubspec.yaml",
    );
    final pubspec = Pubspec.parse(fileContent);
    return pubspec.version!.canonicalizedVersion;
  }

  void processMessage(String payload) {
    try {
      final message = jsonDecode(payload) as Map;
      switch (message['command']) {
        case 'cancel':
          logger('cardScanCancel');
          widget.properties.onCancel?.call();
          break;
        case 'success':
          logger('cardScanSuccess');
          final cardJson = message['card'];
          final card =
              cardJson is Map ? CardResponse.fromJson(cardJson).toCard() : null;
          if (card != null) {
            widget.properties.onSuccess?.call(card);
          } else {
            widget.properties.onError?.call(null);
          }
          break;
        case 'error':
          final error = message['error'];
          final errorMessage = error is Map ? error['message'] : null;
          logger("cardScanError - $error");
          widget.properties.onError?.call(errorMessage);
          break;
        case 'retry':
          logger('cardScanRetry');
          widget.properties.onRetry?.call();
          break;
        case 'syserror':
        case 'console.log':
        case 'console.error':
        case 'console.debug':
          var error = message['message'];
          logger("WebView - $error");
          break;
        default:
          logger("Unknown command received from JS - $message");
      }
    } catch (e, stack) {
      logger(
        "Failed to parse JavascriptMessage - $payload",
        error: e,
        stackTrace: stack,
      );
    }
  }

  Future<void> shouldEnableAndroidDebug() async {
    if (!kIsWeb &&
        Platform.isAndroid &&
        widget.properties.logging == LogLevel.verbose) {
      await InAppWebViewController.setWebContentsDebuggingEnabled(true);
    }
  }

  @override
  Widget build(BuildContext context) {
    return FutureBuilder(
      future: shouldEnableAndroidDebug().then((_) => getPackageVersion()),
      builder: (context, snapshot) {
        if (snapshot.connectionState != ConnectionState.done) {
          return const Center(
            child: CircularProgressIndicator(),
          );
        }

        if (!snapshot.hasData || (snapshot.data?.isEmpty ?? true)) {
          return Center(
            child: TextButton(
              onPressed: () => setState(() {}),
              child: const Text('Reload scanner'),
            ),
          );
        }

        return InAppWebView(
          key: webViewKey,
          initialFile: 'packages/insurance_card_scanner/assets/index.html',
          initialSettings: InAppWebViewSettings(
            useShouldOverrideUrlLoading: true,
            mediaPlaybackRequiresUserGesture: false,
            useHybridComposition: true,
            allowsLinkPreview: false,
            isInspectable: widget.properties.logging == LogLevel.verbose,
            allowsInlineMediaPlayback: true,
          ),
          shouldOverrideUrlLoading: (controller, navigationAction) async {
            return NavigationActionPolicy.ALLOW;
          },
          onWebViewCreated: (controller) {
            logger("onWebViewCreated");

            controller.addJavaScriptHandler(
              handlerName: 'messageHandler',
              callback: (args) {
                logger("messageHandler: $args");
                for (var message in args) {
                  processMessage(message);
                }
              },
            );
          },
          onLoadStop: (controller, url) {
            logger("onLoadStop: $url");
            controller.evaluateJavascript(
              source: jsSnippet(widget.properties),
            );
          },
          shouldInterceptRequest: (controller, request) async {
            logger('intercepting - ${request.url}');
            final url = request.url;
            final scheme = url.scheme;
            final host = url.host;
            if (scheme == 'https' &&
                host.contains('cardscan-models.s3-accelerate.amazonaws.com')) {
              final path = url.path.substring(1); // Remove leading '/'
              logger("interceptRequest - $path");
              try {
                final inputStream = await rootBundle.load('assets/$path');
                final mimeType = path.endsWith(".json")
                    ? "application/json"
                    : "application/octet-stream";
                final response = WebResourceResponse(
                  contentType: mimeType,
                  contentEncoding: 'UTF-8',
                  statusCode: 200,
                  reasonPhrase: 'OK',
                  headers: {
                    "Access-Control-Allow-Origin": "*",
                    "Access-Control-Allow-Methods": "GET",
                    "Content-Type": mimeType
                  },
                  data: inputStream.buffer.asUint8List(),
                );
                logger("interceptRequest - $inputStream");
                return response;
              } catch (e) {
                logger('interceptRequest - failure', error: e);
              }
            }

            return null;
          },
          onPermissionRequest: (_, request) async {
            if (request.resources[0] == PermissionResourceType.CAMERA &&
                request.origin.scheme == 'file') {
              return PermissionResponse(
                resources: request.resources,
                action: PermissionResponseAction.GRANT,
              );
            }
            return PermissionResponse(
              resources: request.resources,
              action: PermissionResponseAction.DENY,
            );
          },
          onReceivedError: (_, request, error) {
            logger("onLoadError: ${error.description}");
            widget.properties.onWebViewLoadError?.call();
          },
          onConsoleMessage: (_, consoleMessage) => logger(consoleMessage),
        );
      },
    );
  }

  @override
  void dispose() {
    super.dispose();
    webViewKey.currentState?.dispose();
  }
}
